module button_filter 
  ( input  wire CLK, RST, BTN,
    output wire TRIGGER );

   parameter ButtonFilterCount = 20_000_000;
   wire    BTN_INT;

   double_flop df
     (.CLK(CLK), .IN(BTN), .OUT(BTN_INT));  

   push_to_pulse # ( .ButtonFilterCount(ButtonFilterCount)) ptp
     (.CLK(CLK), .RST(RST), .BTN(BTN_INT), 
      .TRIG(TRIGGER));

endmodule

module push_to_pulse 
  ( input wire CLK, RST, BTN, 
    output reg TRIG );

   parameter ButtonFilterCount = 20_000_000;

   reg [24:0]  CNT;

   always @ (posedge CLK) begin
      if (RST) begin
	 CNT  <= 0;
	 TRIG <= 0;
      end else begin
	 if (CNT==0) begin
            if (BTN) begin
               CNT  <= ButtonFilterCount;
               TRIG <= 1;
            end
	 end else begin
            CNT  <= CNT-1;
            TRIG <= 0;
	 end
      end
   end
endmodule // push_to_pulse

module double_flop 
  ( input  wire CLK, IN,
    output wire OUT );  

   reg 	   FF1, FF2;

   always @ (posedge CLK) begin
      FF1 <= IN;
      FF2 <= FF1;
   end 

   assign OUT = FF2;

endmodule

